@@ -1,23 +1,30 @@
 # -*- coding: utf-8 -*-
 
-import os
 from AccessControl import Unauthorized
-
-from collective.documentgenerator.config import HAS_PLONE_5
+from appy.shared.zip import unzip
+from collective.documentgenerator.config import get_column_modifier
 from collective.documentgenerator.config import get_raiseOnError_for_non_managers
+from collective.documentgenerator.config import HAS_PLONE_5
+from collective.documentgenerator.config import set_column_modifier
+from collective.documentgenerator.content.pod_template import MailingLoopTemplate
 from collective.documentgenerator.content.pod_template import SubTemplate
+from collective.documentgenerator.events.styles_events import create_temporary_file
 from collective.documentgenerator.interfaces import CyclicMergeTemplatesException
 from collective.documentgenerator.interfaces import PODTemplateNotFoundError
-from collective.documentgenerator.testing import EXAMPLE_POD_TEMPLATE_INTEGRATION
+from collective.documentgenerator.testing import POD_TEMPLATE_INTEGRATION
 from collective.documentgenerator.testing import PODTemplateIntegrationTest
 from collective.documentgenerator.testing import TEST_INSTALL_INTEGRATION
 from collective.documentgenerator.utils import translate as _
-
 from plone import api
 from plone.app.testing import login
 from plone.app.testing import TEST_USER_NAME
+from plone.app.textfield.value import RichTextValue
 from plone.namedfile.file import NamedBlobFile
+from Products.CMFPlone.utils import base_hasattr
+from zope.annotation.interfaces import IAnnotations
 
+import os
+import tempfile
 import unittest
 
 
@@ -109,7 +116,7 @@ class TestGenerationViewMethods(PODTemplateIntegrationTest):
         with self.assertRaises(Exception) as cm:
             view(template_uid, output_format='')
         self.assertEqual(
-            cm.exception.message,
+            str(cm.exception),
             "No 'output_format' found to generate this document"
         )
         # if 'output_format' is not in available pod_formats of template, it raises an Exception
@@ -118,7 +125,7 @@ class TestGenerationViewMethods(PODTemplateIntegrationTest):
         with self.assertRaises(Exception) as cm:
             view(template_uid, 'pdf')
         self.assertEqual(
-            cm.exception.message,
+            str(cm.exception),
             "Asked output format 'pdf' is not available for template 'test_template'!"
         )
 
@@ -155,14 +162,16 @@ class TestGenerationViewMethods(PODTemplateIntegrationTest):
         template_uid = pod_template.UID()
         generation_context = api.content.create(
             type='Folder',
-            title='Folder title long enough to test filename generation with accent héhé',
+            title=u'Folder title > 200m² to test – filename generation 1.2.3.4 with accent héhé '
+                  u'actually we test with a title that is longer than 120 characters so we may '
+                  u'evaluate cropText method that is sensitive to some characters like -',
             id='test_folder',
             container=self.portal,
         )
         generation_view = generation_context.restrictedTraverse('@@persistent-document-generation')
         generation_view(template_uid, 'odt')
-        generated_id = 'general-template-folder-title-long-enough-to-test-filename-generation-with-accent-hehe'
-        msg = "File  {0} should have been created in folder.".format(generated_id)
+        generated_id = 'general-template'
+        msg = "File {0} should have been created in folder.".format(generated_id)
         self.assertTrue(generated_id in generation_context.objectIds(), msg)
 
         persistent_doc = generation_context.get(generated_id)
@@ -178,7 +187,7 @@ class TestGenerationViewMethods(PODTemplateIntegrationTest):
 
         self.assertEqual(
             filename,
-            u'General template - Folder title long enough to test filename generation with accent hehe.odt')
+            u'General template.odt')
         self.assertEqual(content_type, 'application/vnd.oasis.opendocument.text')
 
     def test_persistent_document_generation_on_non_folderish_context(self):
@@ -205,6 +214,7 @@ class TestGenerationViewMethods(PODTemplateIntegrationTest):
     def test__get_generation_context(self):
         pod_template = self.portal.podtemplates['test_template_bis']
         view = self.portal.podtemplates.restrictedTraverse('@@document-generation')
+        view(template_uid=pod_template.UID(), output_format='odt')
         hpv = view.get_generation_context_helper()
         # Check context variables
         self.assertDictEqual(view._get_generation_context(hpv, pod_template), {'details': '1',
@@ -231,7 +241,7 @@ class TestGenerationViewMethods(PODTemplateIntegrationTest):
         # We call rendering to get new gen_context
         rendered, filename, gen_context = view._generate_doc(pod_template, 'odt')
         self.assertIsInstance(gen_context['header'], str)
-        self.assertRegexpMatches(gen_context['header'], '.+(\.odt)$')
+        self.assertRegexpMatches(gen_context['header'], r'.+(\.odt)$')
 
     def test_raiseOnError_for_non_managers(self):
         # create a POD template that will fail in every case
@@ -289,15 +299,233 @@ class TestGenerationViewMethods(PODTemplateIntegrationTest):
         with self.assertRaises(Exception) as cm:
             view(template_uid=template_UID, output_format='odt')
         self.assertTrue(
-            u'Error while evaluating expression "view.unknown_method()".' in
-            cm.exception.message)
+            'Error while evaluating expression "view.unknown_method()".' in str(cm.exception))
+
+    def test_mailing_loop_persistent_document_generation(self):
+        pod_template = self.portal.podtemplates.get('test_template_possibly_mailed')
+        template_uid = pod_template.UID()
+        folder = api.content.create(type='Folder', title=u'Folder', id='test_folder', container=self.portal)
+        # Check dexterity values
+        orig_registry = api.portal.get_registry_record('collective.documentgenerator.mailing_list')
+
+        def get_content(blob_file):
+            tmpdir = tempfile.mkdtemp()
+            temp_file = create_temporary_file(blob_file)
+            info = unzip(temp_file.name, tmpdir, odf=True)
+            os.remove(temp_file.name)
+            return info
+
+        # First we generate a persistent document with only one element in mailing list
+        api.portal.set_registry_record('collective.documentgenerator.mailing_list', orig_registry[0:1])
+        generation_view = folder.restrictedTraverse('@@persistent-document-generation')
+        generation_view(template_uid, 'odt')
+        generated_id = 'possibly-mailed-template'
+        msg = "File {0} should have been created in folder.".format(generated_id)
+        self.assertTrue(generated_id in folder.objectIds(), msg)
+        persistent_doc = folder.get(generated_id)
+        annot = IAnnotations(persistent_doc)
+        self.assertIn('documentgenerator', annot)
+        self.assertNotIn('template_uid', annot['documentgenerator'])
+        self.assertEqual(annot['documentgenerator']['need_mailing'], False)
+        if HAS_PLONE_5:
+            generated_doc = persistent_doc.file
+        else:
+            generated_doc = persistent_doc.getFile()
+        info = get_content(generated_doc)
+        self.assertNotIn('mailed_data', info['content.xml'])
+        self.assertIn('General template', info['content.xml'])
+        self.assertIn('test_template', info['content.xml'])
+
+        # Secondly we generate a persistent document with multiple element in mailing list
+        api.portal.set_registry_record('collective.documentgenerator.mailing_list', orig_registry)
+        generation_view = folder.restrictedTraverse('@@persistent-document-generation')
+        generation_view(template_uid, 'odt')
+        generated_id = 'possibly-mailed-template-1'
+        msg = "File {0} should have been created in folder.".format(generated_id)
+        self.assertTrue(generated_id in folder.objectIds(), msg)
+        persistent_doc = folder.get(generated_id)
+        annot = IAnnotations(persistent_doc)
+        self.assertIn('documentgenerator', annot)
+        self.assertIn('template_uid', annot['documentgenerator'])
+        self.assertIn('context_uid', annot['documentgenerator'])
+        self.assertEqual(annot['documentgenerator']['need_mailing'], True)
+        if HAS_PLONE_5:
+            generated_doc = persistent_doc.file
+        else:
+            generated_doc = persistent_doc.getFile()
+        info = get_content(generated_doc)
+        self.assertNotIn('General template', info['content.xml'])
+        self.assertNotIn('test_template', info['content.xml'])
+        self.assertIn('mailed_data.title', info['content.xml'])
+        self.assertIn('mailed_data.id', info['content.xml'])
+        # check context variables
+        gen_context = generation_view._get_generation_context(generation_view.get_generation_context_helper(),
+                                                              pod_template)
+        self.assertIn('details', gen_context)
+        self.assertEqual(gen_context['details'], '1')
+
+        # the mailing loop view is called on the folder context !
+        generation_view = folder.restrictedTraverse('@@mailing-loop-persistent-document-generation')
+        generation_view(document_uid=persistent_doc.UID())
+        generation_view(document_url_path=persistent_doc.absolute_url_path())
+        generated_id = 'mailing-loop-template'
+        generated_id_2 = 'mailing-loop-template-1'
+        msg = "File {0} should have been created in folder.".format(generated_id)
+        self.assertTrue(generated_id in folder.objectIds(), msg)
+        msg = "File {0} should have been created in folder.".format(generated_id_2)
+        self.assertTrue(generated_id_2 in folder.objectIds(), msg)
+        persistent_doc = folder.get(generated_id)
+        annot = IAnnotations(persistent_doc)
+        self.assertIn('documentgenerator', annot)
+        self.assertNotIn('template_uid', annot['documentgenerator'])
+        self.assertEqual(annot['documentgenerator']['need_mailing'], False)
+        if HAS_PLONE_5:
+            generated_doc = persistent_doc.file
+        else:
+            generated_doc = persistent_doc.getFile()
+        info = get_content(generated_doc)
+        self.assertNotIn('mailed_data', info['content.xml'])
+        self.assertIn('General template', info['content.xml'])
+        self.assertIn('test_template', info['content.xml'])
+        self.assertIn('Multiple format template', info['content.xml'])
+        self.assertIn('test_template_multiple', info['content.xml'])
+        self.assertIn('Collection template', info['content.xml'])
+        self.assertIn('test_template_bis', info['content.xml'])
+        # check that context variables from original template are also in mailing generation context
+        self.assertTrue(isinstance(generation_view.pod_template, MailingLoopTemplate))
+        gen_context = generation_view._get_generation_context(generation_view.get_generation_context_helper(),
+                                                              generation_view.pod_template)
+        self.assertIn('details', gen_context)
+        self.assertEqual(gen_context['details'], '1')
+
+    def test_table_column_modifier(self):
+        """ """
+        pod_template = self.portal.podtemplates.get('test_template_multiple')
+        # do not limit portal_type the POD template is generatable on
+        pod_template.pod_portal_types = []
+        template_uid = pod_template.UID()
+        doc = self.portal.get('front-page')
+        # add a table to check if it is optimized or not
+        text = u'<table><tr><td>Text1</td><td>Text2</td></tr><tr><td>Text3</td><td>Text4</td></tr></table>'
+        textNone = u'<table style="table-layout:none"><tr><td>Text1</td><td>Text2</td></tr><tr><td>Text3</td><td>Text4</td></tr></table>'
+        textAuto = u'<table style="table-layout:auto"><tr><td>Text1</td><td>Text2</td></tr><tr><td>Text3</td><td>Text4</td></tr></table>'
+        textFixed = u'<table style="table-layout:fixed"><tr><td>Text1</td><td>Text2</td></tr><tr><td>Text3</td><td>Text4</td></tr></table>'
+
+        generation_view = doc.restrictedTraverse('@@document-generation')
+
+        def set_text(text):
+            if base_hasattr(doc, 'setText'):
+                # Archetypes
+                doc.setText(text)
+            else:
+                # Dexterity
+                doc.text = RichTextValue(text)
+
+        def assert_result(ocw_in_xml, dc_in_xml):
+            generated_doc = generation_view(template_uid, 'odt')
+            content_xml = self.get_odt_content_xml(generated_doc)
+            self.assertEquals(ocw_in_xml, 'OCW' in content_xml)
+            self.assertEquals(dc_in_xml, 'DC' in content_xml)
+
+        # By default : column_modifier disabled globally, CSS override enabled globally and pod_template using global parameter
+        self.assertEquals(get_column_modifier(), 'nothing')
+        self.assertEquals(pod_template.get_column_modifier(), -1)
+
+        set_text(text)
+        assert_result(False, False)
+        set_text(textNone)
+        assert_result(False, False)
+        set_text(textAuto)
+        assert_result(True, False)
+        set_text(textFixed)
+        assert_result(False, True)
+
+        # disable optimize locally, still disabled globally
+        pod_template.column_modifier = 'disabled'
+        set_text(text)
+        assert_result(False, False)
+        set_text(textNone)
+        assert_result(False, False)
+        set_text(textAuto)
+        assert_result(False, False)
+        set_text(textFixed)
+        assert_result(False, False)
+
+        # enable optimize locally, still disabled globally
+        pod_template.column_modifier = 'optimize'
+        set_text(text)
+        assert_result(True, False)
+        set_text(textNone)
+        assert_result(False, False)
+        set_text(textAuto)
+        assert_result(True, False)
+        set_text(textFixed)
+        assert_result(False, True)
+
+        # enable distribute locally, still disabled globally
+        pod_template.column_modifier = 'distribute'
+        set_text(text)
+        assert_result(False, True)
+        set_text(textNone)
+        assert_result(False, False)
+        set_text(textAuto)
+        assert_result(True, False)
+        set_text(textFixed)
+        assert_result(False, True)
+
+        # reset local parameter
+        pod_template.column_modifier = -1
+
+        # CSS override enabled
+        set_column_modifier('nothing')
+        set_text(text)
+        assert_result(False, False)
+        set_text(textNone)
+        assert_result(False, False)
+        set_text(textAuto)
+        assert_result(True, False)
+        set_text(textFixed)
+        assert_result(False, True)
+
+        # disable all column modifier globally
+        set_column_modifier('disabled')
+        set_text(text)
+        assert_result(False, False)
+        set_text(textNone)
+        assert_result(False, False)
+        set_text(textAuto)
+        assert_result(False, False)
+        set_text(textFixed)
+        assert_result(False, False)
+
+        # optimize column_modifier enabled globally and pod_template using global parameter
+        set_column_modifier('optimize')
+        set_text(text)
+        assert_result(True, False)
+        set_text(textNone)
+        assert_result(False, False)
+        set_text(textAuto)
+        assert_result(True, False)
+        set_text(textFixed)
+        assert_result(False, True)
+
+        # distribute column_modifier enabled globally and pod_template using global parameter
+        set_column_modifier('distribute')
+        set_text(text)
+        assert_result(False, True)
+        set_text(textNone)
+        assert_result(False, False)
+        set_text(textAuto)
+        assert_result(True, False)
+        set_text(textFixed)
+        assert_result(False, True)
 
 
 class TestCyclicMergesDetection(unittest.TestCase):
     """
     """
 
-    layer = EXAMPLE_POD_TEMPLATE_INTEGRATION
+    layer = POD_TEMPLATE_INTEGRATION
 
     def setUp(self):
         super(TestCyclicMergesDetection, self).setUp()
